@@ -79,7 +79,7 @@
 	      \"Ripples (stir)\"	ripples -root -oily -light 2 -stir	    \\n\
 	   \"Ripples (desktop)\"	ripples -root -water -light 6		    \\n\
 				hypercube -root				    \\n\
-				hyperball -root				    \\n\
+-				hyperball -root				    \\n\
 				halo -root				    \\n\
 				maze -root				    \\n\
 				noseguy -root				    \\n\
@@ -219,7 +219,7 @@
 -	   GL:	   \"Pulsar (textures)\"					      \
 				  pulsar -root -texture -mipmap		      \
 				  -texture_quality -light -fog		    \\n\
-	   GL:			extrusion -root				    \\n\
+-	   GL:			extrusion -root				    \\n\
 	   GL:			sierpinski3d -root			    \\n\
 	   GL:			menger -root				    \\n\
 	   GL:	 \"GFlux\"	gflux -root				    \\n\
@@ -250,6 +250,15 @@
 	   GL:			bouncingcow -root			    \\n\
 	   GL:			jigglypuff -root -random		    \\n\
 	   GL:			klein -root -random			    \\n\
+	   GL:	\"HyperTorus (striped)\"	hypertorus -root		    \\n\
+	   GL:	\"HyperTorus (solid)\"	hypertorus -root -solid -transp	    \\n\
+	   GL:			glmatrix -root				    \\n\
+	   GL:			cubestorm -root				    \\n\
+	   GL:			glknots -root				    \\n\
+	   GL:			blocktube -root				    \\n\
+	   GL:			flipflop -root				    \\n\
+	   GL:			antspotlight -root			    \\n\
+	   GL:			polytopes -root				    \\n\
 -	   GL:			glslideshow -root			    \\n\
 									      \
 -				xdaliclock -root -builtin3 -cycle	    \\n\
@@ -268,7 +277,8 @@
 -				goban -root                                 \\n\
 -				electricsheep                               \\n\
 -				cosmos -root                                \\n\
--	   GL:                  sphereEversion --root                       \\n",
+-	   GL:                  sphereEversion --root                       \\n\
+-	   GL:                  fireflies -root                             \\n",
 "XScreenSaver.pointerPollTime:		0:00:05",
 "XScreenSaver.initialDelay:		0:00:00",
 "XScreenSaver.windowCreationTimeout:	0:00:30",
@@ -431,7 +441,6 @@
 "*hacks.vidwhacker.name:     VidWhacker",
 "*hacks.morph3d.name:        Morph3D",
 "*hacks.bubble3d.name:       Bubble3D",
-"*hacks.glplanet.name:       GLPlanet",
 "*hacks.sierpinski3d.name:   Sierpinski3D",
 "*hacks.gflux.name:          GFlux",
 "*hacks.xrayswarm.name:      XRaySwarm",
@@ -439,12 +448,9 @@
 "*hacks.rotzoomer.name:      RotZoomer",
 "*hacks.stonerview.name:     StonerView",
 "*hacks.starwars.name:       StarWars",
-"*hacks.glblur.name:         GLBlur",
-"*hacks.gltext.name:         GLText",
 "*hacks.dangerball.name:     DangerBall",
 "*hacks.whirlygig.name:      WhirlyGig",
 "*hacks.speedmine.name:      SpeedMine",
-"*hacks.glsnake.name:        GLSnake",
 "*hacks.glforestfire.name:   GLForestFire",
 "*hacks.sballs.name:         SBalls",
 "*hacks.xdaliclock.name:     XDaliClock",
@@ -457,5 +463,9 @@
 "*hacks.flyingtoasters.name: FlyingToasters",
 "*hacks.bouncingcow.name:    BouncingCow",
 "*hacks.jigglypuff.name:     JigglyPuff",
-"*hacks.glslideshow.name:    GLSlideshow",
+"*hacks.hypertorus.name:     HyperTorus",
+"*hacks.cubestorm.name:      CubeStorm",
+"*hacks.blocktube.name:      BlockTube",
+"*hacks.flipflop.name:       FlipFlop",
+"*hacks.antspotlight.name:   AntSpotlight",
 "*hacks.documentation.isInstalled: True",
